//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Mask/MaskContainerModel.cpp
//! @brief     Implements MaskItems classes
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Model/Mask/MaskContainerModel.h"
#include "Base/Util/Assert.h"
#include "GUI/Model/Mask/MaskContainerItem.h"
#include "GUI/Model/Mask/MaskItems.h"

// Implementation of MaskContainerModel is based on the Qt source code for QStringListModel

MaskContainerModel::MaskContainerModel(MaskContainerItem* container)
    : maskContainer(container)
{
}

MaskContainerModel::~MaskContainerModel() = default;

// Begin overridden methods from QAbstractListModel

int MaskContainerModel::rowCount(const QModelIndex& parent) const
{
    if (parent.isValid())
        return 0;

    return maskContainer->size();
}

QVariant MaskContainerModel::data(const QModelIndex& index, int role) const
{
    const qsizetype row = index.row();

    if (row < 0 || row >= maskContainer->size())
        return {};

    if (role == Qt::DisplayRole || role == Qt::EditRole)
        return {maskContainer->at(row)->maskName()};

    return {};
}

// End overridden methods from QAbstractListModel

void MaskContainerModel::insertMask(int row, MaskItem* maskItem)
{
    QAbstractListModel::beginInsertRows(maskContainer->rootIndex, row, row);
    maskContainer->insertMask(row, maskItem);
    QAbstractListModel::endInsertRows();
}

void MaskContainerModel::maskToModel(MaskItem* maskItem)
{
    qsizetype row = maskContainer->size() - 1;
    QAbstractListModel::beginInsertRows(maskContainer->rootIndex, row, row);
    maskContainer->addMaskItem(maskItem);
    QAbstractListModel::endInsertRows();
}


//! Move mask to a given row
void MaskContainerModel::moveMask(int from_row, int to_row)
{
    emit QAbstractListModel::beginMoveRows(maskContainer->rootIndex, from_row, from_row,
                                           maskContainer->rootIndex, to_row);
    maskContainer->moveMask(from_row, to_row);
    emit QAbstractListModel::endMoveRows();
}

void MaskContainerModel::removeMaskAt(int row)
{
    QAbstractListModel::beginRemoveRows(maskContainer->rootIndex, row, row);
    maskContainer->removeMaskAt(row);
    QAbstractListModel::endRemoveRows();
}

void MaskContainerModel::removeMask(MaskItem* maskItem)
{
    const int row = maskContainer->indexOfItem(maskItem);
    removeMaskAt(row);
}

RegionOfInterestItem* MaskContainerModel::regionOfInterestItem() const
{
    for (MaskItem* maskItem : maskContainer->maskItems())
        if (auto* reg = dynamic_cast<RegionOfInterestItem*>(maskItem))
            return reg;

    return nullptr;
}

QModelIndex MaskContainerModel::indexOfItem(MaskRoot* item) const
{
    const int row = maskContainer->indexOfItem(dynamic_cast<MaskItem*>(item));
    return QAbstractListModel::index(row, 0, {});
}

MaskItem* MaskContainerModel::itemForIndex(const QModelIndex& index) const
{
    if (index.isValid())
        return maskContainer->at(index.row());

    return nullptr;
}

void MaskContainerModel::clear()
{
    QAbstractListModel::beginResetModel();
    maskContainer->clear();
    QAbstractListModel::endResetModel();
}

void MaskContainerModel::copy(const MaskContainerModel* src)
{
    clear();
    if (src)
        for (auto mask : src->maskContainer->maskItems())
            maskToModel(mask);
}
